/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.All
{
    /// <summary>
    /// TpdmCandidate
    /// </summary>
    [DataContract(Name = "tpdm_candidate")]
    public partial class TpdmCandidate : IEquatable<TpdmCandidate>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TpdmCandidate" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected TpdmCandidate() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="TpdmCandidate" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="candidateIdentifier">A unique alphanumeric code assigned to a candidate. (required).</param>
        /// <param name="personReference">personReference.</param>
        /// <param name="addresses">An unordered collection of candidateAddresses. The set of elements that describes an address, including the street address, city, state, and ZIP code..</param>
        /// <param name="birthCity">The city the student was born in..</param>
        /// <param name="birthCountryDescriptor">The country in which an individual is born. It is strongly recommended that entries use only ISO 3166 2-letter country codes..</param>
        /// <param name="birthDate">The month, day, and year on which an individual was born. (required).</param>
        /// <param name="birthInternationalProvince">For students born outside of the U.S., the Province or jurisdiction in which an individual is born..</param>
        /// <param name="birthSexDescriptor">A person&#39;s gender at birth..</param>
        /// <param name="birthStateAbbreviationDescriptor">The abbreviation for the name of the state (within the United States) or extra-state jurisdiction in which an individual was born..</param>
        /// <param name="dateEnteredUS">For students born outside of the U.S., the date the student entered the U.S..</param>
        /// <param name="disabilities">An unordered collection of candidateDisabilities. The disability condition(s) that best describes an individual&#39;s impairment..</param>
        /// <param name="displacementStatus">Indicates a state health or weather related event that displaces a group of students, and may require additional funding, educational, or social services..</param>
        /// <param name="economicDisadvantaged">An indication of inadequate financial condition of an individual&#39;s family, as determined by family income, number of family members/dependents, participation in public assistance programs, and/or other characteristics considered relevant by federal, state, and local policy..</param>
        /// <param name="electronicMails">An unordered collection of candidateElectronicMails. The numbers, letters, and symbols used to identify an electronic mail (e-mail) user within the network to which the individual or organization belongs..</param>
        /// <param name="englishLanguageExamDescriptor">Indicates that a person passed, failed, or did not take an English Language assessment (e.g., TOEFFL)..</param>
        /// <param name="firstGenerationStudent">Indicator of whether individual is a first generation college student..</param>
        /// <param name="firstName">A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change. (required).</param>
        /// <param name="genderDescriptor">The gender of the candidate..</param>
        /// <param name="generationCodeSuffix">An appendage, if any, used to denote an individual&#39;s generation in his family (e.g., Jr., Sr., III)..</param>
        /// <param name="hispanicLatinoEthnicity">An indication that the individual traces his or her origin or descent to Mexico, Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, \&quot;Spanish origin,\&quot; can be used in addition to \&quot;Hispanic or Latino.\&quot;.</param>
        /// <param name="languages">An unordered collection of candidateLanguages. The language(s) the individual uses to communicate..</param>
        /// <param name="lastSurname">The name borne in common by members of a family. (required).</param>
        /// <param name="limitedEnglishProficiencyDescriptor">An indication that the student has been identified as limited English proficient by the Language Proficiency Assessment Committee (LPAC), or English proficient..</param>
        /// <param name="maidenName">The individual&#39;s maiden name..</param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony..</param>
        /// <param name="multipleBirthStatus">Indicator of whether the student was born with other siblings (i.e., twins, triplets, etc.).</param>
        /// <param name="otherNames">An unordered collection of candidateOtherNames. Other names (e.g., alias, nickname, previous legal name) associated with a person..</param>
        /// <param name="personalIdentificationDocuments">An unordered collection of candidatePersonalIdentificationDocuments. The documents presented as evident to verify one&#39;s personal identity; for example: drivers license, passport, birth certificate, etc..</param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the individual..</param>
        /// <param name="races">An unordered collection of candidateRaces. The general racial category which most clearly reflects the individual&#39;s recognition of his or her community or with which the individual most identifies. The data model allows for multiple entries so that each individual can specify all appropriate races..</param>
        /// <param name="sexDescriptor">The sex of the candidate. (required).</param>
        /// <param name="telephones">An unordered collection of candidateTelephones. The 10-digit telephone number, including the area code, for the person..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public TpdmCandidate(string id = default(string), string candidateIdentifier = default(string), EdFiPersonReference personReference = default(EdFiPersonReference), List<TpdmCandidateAddress> addresses = default(List<TpdmCandidateAddress>), string birthCity = default(string), string birthCountryDescriptor = default(string), DateTime birthDate = default(DateTime), string birthInternationalProvince = default(string), string birthSexDescriptor = default(string), string birthStateAbbreviationDescriptor = default(string), DateTime dateEnteredUS = default(DateTime), List<TpdmCandidateDisability> disabilities = default(List<TpdmCandidateDisability>), string displacementStatus = default(string), bool economicDisadvantaged = default(bool), List<TpdmCandidateElectronicMail> electronicMails = default(List<TpdmCandidateElectronicMail>), string englishLanguageExamDescriptor = default(string), bool firstGenerationStudent = default(bool), string firstName = default(string), string genderDescriptor = default(string), string generationCodeSuffix = default(string), bool hispanicLatinoEthnicity = default(bool), List<TpdmCandidateLanguage> languages = default(List<TpdmCandidateLanguage>), string lastSurname = default(string), string limitedEnglishProficiencyDescriptor = default(string), string maidenName = default(string), string middleName = default(string), bool multipleBirthStatus = default(bool), List<TpdmCandidateOtherName> otherNames = default(List<TpdmCandidateOtherName>), List<TpdmCandidatePersonalIdentificationDocument> personalIdentificationDocuments = default(List<TpdmCandidatePersonalIdentificationDocument>), string personalTitlePrefix = default(string), List<TpdmCandidateRace> races = default(List<TpdmCandidateRace>), string sexDescriptor = default(string), List<TpdmCandidateTelephone> telephones = default(List<TpdmCandidateTelephone>), string etag = default(string))
        {
            // to ensure "candidateIdentifier" is required (not null)
            if (candidateIdentifier == null) {
                throw new ArgumentNullException("candidateIdentifier is a required property for TpdmCandidate and cannot be null");
            }
            this.CandidateIdentifier = candidateIdentifier;
            this.BirthDate = birthDate;
            // to ensure "firstName" is required (not null)
            if (firstName == null) {
                throw new ArgumentNullException("firstName is a required property for TpdmCandidate and cannot be null");
            }
            this.FirstName = firstName;
            // to ensure "lastSurname" is required (not null)
            if (lastSurname == null) {
                throw new ArgumentNullException("lastSurname is a required property for TpdmCandidate and cannot be null");
            }
            this.LastSurname = lastSurname;
            // to ensure "sexDescriptor" is required (not null)
            if (sexDescriptor == null) {
                throw new ArgumentNullException("sexDescriptor is a required property for TpdmCandidate and cannot be null");
            }
            this.SexDescriptor = sexDescriptor;
            this.Id = id;
            this.PersonReference = personReference;
            this.Addresses = addresses;
            this.BirthCity = birthCity;
            this.BirthCountryDescriptor = birthCountryDescriptor;
            this.BirthInternationalProvince = birthInternationalProvince;
            this.BirthSexDescriptor = birthSexDescriptor;
            this.BirthStateAbbreviationDescriptor = birthStateAbbreviationDescriptor;
            this.DateEnteredUS = dateEnteredUS;
            this.Disabilities = disabilities;
            this.DisplacementStatus = displacementStatus;
            this.EconomicDisadvantaged = economicDisadvantaged;
            this.ElectronicMails = electronicMails;
            this.EnglishLanguageExamDescriptor = englishLanguageExamDescriptor;
            this.FirstGenerationStudent = firstGenerationStudent;
            this.GenderDescriptor = genderDescriptor;
            this.GenerationCodeSuffix = generationCodeSuffix;
            this.HispanicLatinoEthnicity = hispanicLatinoEthnicity;
            this.Languages = languages;
            this.LimitedEnglishProficiencyDescriptor = limitedEnglishProficiencyDescriptor;
            this.MaidenName = maidenName;
            this.MiddleName = middleName;
            this.MultipleBirthStatus = multipleBirthStatus;
            this.OtherNames = otherNames;
            this.PersonalIdentificationDocuments = personalIdentificationDocuments;
            this.PersonalTitlePrefix = personalTitlePrefix;
            this.Races = races;
            this.Telephones = telephones;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// A unique alphanumeric code assigned to a candidate.
        /// </summary>
        /// <value>A unique alphanumeric code assigned to a candidate.</value>
        [DataMember(Name = "candidateIdentifier", IsRequired = true, EmitDefaultValue = false)]
        public string CandidateIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets PersonReference
        /// </summary>
        [DataMember(Name = "personReference", EmitDefaultValue = false)]
        public EdFiPersonReference PersonReference { get; set; }

        /// <summary>
        /// An unordered collection of candidateAddresses. The set of elements that describes an address, including the street address, city, state, and ZIP code.
        /// </summary>
        /// <value>An unordered collection of candidateAddresses. The set of elements that describes an address, including the street address, city, state, and ZIP code.</value>
        [DataMember(Name = "addresses", EmitDefaultValue = false)]
        public List<TpdmCandidateAddress> Addresses { get; set; }

        /// <summary>
        /// The city the student was born in.
        /// </summary>
        /// <value>The city the student was born in.</value>
        [DataMember(Name = "birthCity", EmitDefaultValue = false)]
        public string BirthCity { get; set; }

        /// <summary>
        /// The country in which an individual is born. It is strongly recommended that entries use only ISO 3166 2-letter country codes.
        /// </summary>
        /// <value>The country in which an individual is born. It is strongly recommended that entries use only ISO 3166 2-letter country codes.</value>
        [DataMember(Name = "birthCountryDescriptor", EmitDefaultValue = false)]
        public string BirthCountryDescriptor { get; set; }

        /// <summary>
        /// The month, day, and year on which an individual was born.
        /// </summary>
        /// <value>The month, day, and year on which an individual was born.</value>
        [DataMember(Name = "birthDate", IsRequired = true, EmitDefaultValue = false)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime BirthDate { get; set; }

        /// <summary>
        /// For students born outside of the U.S., the Province or jurisdiction in which an individual is born.
        /// </summary>
        /// <value>For students born outside of the U.S., the Province or jurisdiction in which an individual is born.</value>
        [DataMember(Name = "birthInternationalProvince", EmitDefaultValue = false)]
        public string BirthInternationalProvince { get; set; }

        /// <summary>
        /// A person&#39;s gender at birth.
        /// </summary>
        /// <value>A person&#39;s gender at birth.</value>
        [DataMember(Name = "birthSexDescriptor", EmitDefaultValue = false)]
        public string BirthSexDescriptor { get; set; }

        /// <summary>
        /// The abbreviation for the name of the state (within the United States) or extra-state jurisdiction in which an individual was born.
        /// </summary>
        /// <value>The abbreviation for the name of the state (within the United States) or extra-state jurisdiction in which an individual was born.</value>
        [DataMember(Name = "birthStateAbbreviationDescriptor", EmitDefaultValue = false)]
        public string BirthStateAbbreviationDescriptor { get; set; }

        /// <summary>
        /// For students born outside of the U.S., the date the student entered the U.S.
        /// </summary>
        /// <value>For students born outside of the U.S., the date the student entered the U.S.</value>
        [DataMember(Name = "dateEnteredUS", EmitDefaultValue = false)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime DateEnteredUS { get; set; }

        /// <summary>
        /// An unordered collection of candidateDisabilities. The disability condition(s) that best describes an individual&#39;s impairment.
        /// </summary>
        /// <value>An unordered collection of candidateDisabilities. The disability condition(s) that best describes an individual&#39;s impairment.</value>
        [DataMember(Name = "disabilities", EmitDefaultValue = false)]
        public List<TpdmCandidateDisability> Disabilities { get; set; }

        /// <summary>
        /// Indicates a state health or weather related event that displaces a group of students, and may require additional funding, educational, or social services.
        /// </summary>
        /// <value>Indicates a state health or weather related event that displaces a group of students, and may require additional funding, educational, or social services.</value>
        [DataMember(Name = "displacementStatus", EmitDefaultValue = false)]
        public string DisplacementStatus { get; set; }

        /// <summary>
        /// An indication of inadequate financial condition of an individual&#39;s family, as determined by family income, number of family members/dependents, participation in public assistance programs, and/or other characteristics considered relevant by federal, state, and local policy.
        /// </summary>
        /// <value>An indication of inadequate financial condition of an individual&#39;s family, as determined by family income, number of family members/dependents, participation in public assistance programs, and/or other characteristics considered relevant by federal, state, and local policy.</value>
        [DataMember(Name = "economicDisadvantaged", EmitDefaultValue = true)]
        public bool EconomicDisadvantaged { get; set; }

        /// <summary>
        /// An unordered collection of candidateElectronicMails. The numbers, letters, and symbols used to identify an electronic mail (e-mail) user within the network to which the individual or organization belongs.
        /// </summary>
        /// <value>An unordered collection of candidateElectronicMails. The numbers, letters, and symbols used to identify an electronic mail (e-mail) user within the network to which the individual or organization belongs.</value>
        [DataMember(Name = "electronicMails", EmitDefaultValue = false)]
        public List<TpdmCandidateElectronicMail> ElectronicMails { get; set; }

        /// <summary>
        /// Indicates that a person passed, failed, or did not take an English Language assessment (e.g., TOEFFL).
        /// </summary>
        /// <value>Indicates that a person passed, failed, or did not take an English Language assessment (e.g., TOEFFL).</value>
        [DataMember(Name = "englishLanguageExamDescriptor", EmitDefaultValue = false)]
        public string EnglishLanguageExamDescriptor { get; set; }

        /// <summary>
        /// Indicator of whether individual is a first generation college student.
        /// </summary>
        /// <value>Indicator of whether individual is a first generation college student.</value>
        [DataMember(Name = "firstGenerationStudent", EmitDefaultValue = true)]
        public bool FirstGenerationStudent { get; set; }

        /// <summary>
        /// A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.
        /// </summary>
        /// <value>A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.</value>
        [DataMember(Name = "firstName", IsRequired = true, EmitDefaultValue = false)]
        public string FirstName { get; set; }

        /// <summary>
        /// The gender of the candidate.
        /// </summary>
        /// <value>The gender of the candidate.</value>
        [DataMember(Name = "genderDescriptor", EmitDefaultValue = false)]
        public string GenderDescriptor { get; set; }

        /// <summary>
        /// An appendage, if any, used to denote an individual&#39;s generation in his family (e.g., Jr., Sr., III).
        /// </summary>
        /// <value>An appendage, if any, used to denote an individual&#39;s generation in his family (e.g., Jr., Sr., III).</value>
        [DataMember(Name = "generationCodeSuffix", EmitDefaultValue = false)]
        public string GenerationCodeSuffix { get; set; }

        /// <summary>
        /// An indication that the individual traces his or her origin or descent to Mexico, Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, \&quot;Spanish origin,\&quot; can be used in addition to \&quot;Hispanic or Latino.\&quot;
        /// </summary>
        /// <value>An indication that the individual traces his or her origin or descent to Mexico, Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, \&quot;Spanish origin,\&quot; can be used in addition to \&quot;Hispanic or Latino.\&quot;</value>
        [DataMember(Name = "hispanicLatinoEthnicity", EmitDefaultValue = true)]
        public bool HispanicLatinoEthnicity { get; set; }

        /// <summary>
        /// An unordered collection of candidateLanguages. The language(s) the individual uses to communicate.
        /// </summary>
        /// <value>An unordered collection of candidateLanguages. The language(s) the individual uses to communicate.</value>
        [DataMember(Name = "languages", EmitDefaultValue = false)]
        public List<TpdmCandidateLanguage> Languages { get; set; }

        /// <summary>
        /// The name borne in common by members of a family.
        /// </summary>
        /// <value>The name borne in common by members of a family.</value>
        [DataMember(Name = "lastSurname", IsRequired = true, EmitDefaultValue = false)]
        public string LastSurname { get; set; }

        /// <summary>
        /// An indication that the student has been identified as limited English proficient by the Language Proficiency Assessment Committee (LPAC), or English proficient.
        /// </summary>
        /// <value>An indication that the student has been identified as limited English proficient by the Language Proficiency Assessment Committee (LPAC), or English proficient.</value>
        [DataMember(Name = "limitedEnglishProficiencyDescriptor", EmitDefaultValue = false)]
        public string LimitedEnglishProficiencyDescriptor { get; set; }

        /// <summary>
        /// The individual&#39;s maiden name.
        /// </summary>
        /// <value>The individual&#39;s maiden name.</value>
        [DataMember(Name = "maidenName", EmitDefaultValue = false)]
        public string MaidenName { get; set; }

        /// <summary>
        /// A secondary name given to an individual at birth, baptism, or during another naming ceremony.
        /// </summary>
        /// <value>A secondary name given to an individual at birth, baptism, or during another naming ceremony.</value>
        [DataMember(Name = "middleName", EmitDefaultValue = false)]
        public string MiddleName { get; set; }

        /// <summary>
        /// Indicator of whether the student was born with other siblings (i.e., twins, triplets, etc.)
        /// </summary>
        /// <value>Indicator of whether the student was born with other siblings (i.e., twins, triplets, etc.)</value>
        [DataMember(Name = "multipleBirthStatus", EmitDefaultValue = true)]
        public bool MultipleBirthStatus { get; set; }

        /// <summary>
        /// An unordered collection of candidateOtherNames. Other names (e.g., alias, nickname, previous legal name) associated with a person.
        /// </summary>
        /// <value>An unordered collection of candidateOtherNames. Other names (e.g., alias, nickname, previous legal name) associated with a person.</value>
        [DataMember(Name = "otherNames", EmitDefaultValue = false)]
        public List<TpdmCandidateOtherName> OtherNames { get; set; }

        /// <summary>
        /// An unordered collection of candidatePersonalIdentificationDocuments. The documents presented as evident to verify one&#39;s personal identity; for example: drivers license, passport, birth certificate, etc.
        /// </summary>
        /// <value>An unordered collection of candidatePersonalIdentificationDocuments. The documents presented as evident to verify one&#39;s personal identity; for example: drivers license, passport, birth certificate, etc.</value>
        [DataMember(Name = "personalIdentificationDocuments", EmitDefaultValue = false)]
        public List<TpdmCandidatePersonalIdentificationDocument> PersonalIdentificationDocuments { get; set; }

        /// <summary>
        /// A prefix used to denote the title, degree, position, or seniority of the individual.
        /// </summary>
        /// <value>A prefix used to denote the title, degree, position, or seniority of the individual.</value>
        [DataMember(Name = "personalTitlePrefix", EmitDefaultValue = false)]
        public string PersonalTitlePrefix { get; set; }

        /// <summary>
        /// An unordered collection of candidateRaces. The general racial category which most clearly reflects the individual&#39;s recognition of his or her community or with which the individual most identifies. The data model allows for multiple entries so that each individual can specify all appropriate races.
        /// </summary>
        /// <value>An unordered collection of candidateRaces. The general racial category which most clearly reflects the individual&#39;s recognition of his or her community or with which the individual most identifies. The data model allows for multiple entries so that each individual can specify all appropriate races.</value>
        [DataMember(Name = "races", EmitDefaultValue = false)]
        public List<TpdmCandidateRace> Races { get; set; }

        /// <summary>
        /// The sex of the candidate.
        /// </summary>
        /// <value>The sex of the candidate.</value>
        [DataMember(Name = "sexDescriptor", IsRequired = true, EmitDefaultValue = false)]
        public string SexDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of candidateTelephones. The 10-digit telephone number, including the area code, for the person.
        /// </summary>
        /// <value>An unordered collection of candidateTelephones. The 10-digit telephone number, including the area code, for the person.</value>
        [DataMember(Name = "telephones", EmitDefaultValue = false)]
        public List<TpdmCandidateTelephone> Telephones { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class TpdmCandidate {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  CandidateIdentifier: ").Append(CandidateIdentifier).Append("\n");
            sb.Append("  PersonReference: ").Append(PersonReference).Append("\n");
            sb.Append("  Addresses: ").Append(Addresses).Append("\n");
            sb.Append("  BirthCity: ").Append(BirthCity).Append("\n");
            sb.Append("  BirthCountryDescriptor: ").Append(BirthCountryDescriptor).Append("\n");
            sb.Append("  BirthDate: ").Append(BirthDate).Append("\n");
            sb.Append("  BirthInternationalProvince: ").Append(BirthInternationalProvince).Append("\n");
            sb.Append("  BirthSexDescriptor: ").Append(BirthSexDescriptor).Append("\n");
            sb.Append("  BirthStateAbbreviationDescriptor: ").Append(BirthStateAbbreviationDescriptor).Append("\n");
            sb.Append("  DateEnteredUS: ").Append(DateEnteredUS).Append("\n");
            sb.Append("  Disabilities: ").Append(Disabilities).Append("\n");
            sb.Append("  DisplacementStatus: ").Append(DisplacementStatus).Append("\n");
            sb.Append("  EconomicDisadvantaged: ").Append(EconomicDisadvantaged).Append("\n");
            sb.Append("  ElectronicMails: ").Append(ElectronicMails).Append("\n");
            sb.Append("  EnglishLanguageExamDescriptor: ").Append(EnglishLanguageExamDescriptor).Append("\n");
            sb.Append("  FirstGenerationStudent: ").Append(FirstGenerationStudent).Append("\n");
            sb.Append("  FirstName: ").Append(FirstName).Append("\n");
            sb.Append("  GenderDescriptor: ").Append(GenderDescriptor).Append("\n");
            sb.Append("  GenerationCodeSuffix: ").Append(GenerationCodeSuffix).Append("\n");
            sb.Append("  HispanicLatinoEthnicity: ").Append(HispanicLatinoEthnicity).Append("\n");
            sb.Append("  Languages: ").Append(Languages).Append("\n");
            sb.Append("  LastSurname: ").Append(LastSurname).Append("\n");
            sb.Append("  LimitedEnglishProficiencyDescriptor: ").Append(LimitedEnglishProficiencyDescriptor).Append("\n");
            sb.Append("  MaidenName: ").Append(MaidenName).Append("\n");
            sb.Append("  MiddleName: ").Append(MiddleName).Append("\n");
            sb.Append("  MultipleBirthStatus: ").Append(MultipleBirthStatus).Append("\n");
            sb.Append("  OtherNames: ").Append(OtherNames).Append("\n");
            sb.Append("  PersonalIdentificationDocuments: ").Append(PersonalIdentificationDocuments).Append("\n");
            sb.Append("  PersonalTitlePrefix: ").Append(PersonalTitlePrefix).Append("\n");
            sb.Append("  Races: ").Append(Races).Append("\n");
            sb.Append("  SexDescriptor: ").Append(SexDescriptor).Append("\n");
            sb.Append("  Telephones: ").Append(Telephones).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as TpdmCandidate);
        }

        /// <summary>
        /// Returns true if TpdmCandidate instances are equal
        /// </summary>
        /// <param name="input">Instance of TpdmCandidate to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(TpdmCandidate input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.CandidateIdentifier == input.CandidateIdentifier ||
                    (this.CandidateIdentifier != null &&
                    this.CandidateIdentifier.Equals(input.CandidateIdentifier))
                ) && 
                (
                    this.PersonReference == input.PersonReference ||
                    (this.PersonReference != null &&
                    this.PersonReference.Equals(input.PersonReference))
                ) && 
                (
                    this.Addresses == input.Addresses ||
                    this.Addresses != null &&
                    input.Addresses != null &&
                    this.Addresses.SequenceEqual(input.Addresses)
                ) && 
                (
                    this.BirthCity == input.BirthCity ||
                    (this.BirthCity != null &&
                    this.BirthCity.Equals(input.BirthCity))
                ) && 
                (
                    this.BirthCountryDescriptor == input.BirthCountryDescriptor ||
                    (this.BirthCountryDescriptor != null &&
                    this.BirthCountryDescriptor.Equals(input.BirthCountryDescriptor))
                ) && 
                (
                    this.BirthDate == input.BirthDate ||
                    (this.BirthDate != null &&
                    this.BirthDate.Equals(input.BirthDate))
                ) && 
                (
                    this.BirthInternationalProvince == input.BirthInternationalProvince ||
                    (this.BirthInternationalProvince != null &&
                    this.BirthInternationalProvince.Equals(input.BirthInternationalProvince))
                ) && 
                (
                    this.BirthSexDescriptor == input.BirthSexDescriptor ||
                    (this.BirthSexDescriptor != null &&
                    this.BirthSexDescriptor.Equals(input.BirthSexDescriptor))
                ) && 
                (
                    this.BirthStateAbbreviationDescriptor == input.BirthStateAbbreviationDescriptor ||
                    (this.BirthStateAbbreviationDescriptor != null &&
                    this.BirthStateAbbreviationDescriptor.Equals(input.BirthStateAbbreviationDescriptor))
                ) && 
                (
                    this.DateEnteredUS == input.DateEnteredUS ||
                    (this.DateEnteredUS != null &&
                    this.DateEnteredUS.Equals(input.DateEnteredUS))
                ) && 
                (
                    this.Disabilities == input.Disabilities ||
                    this.Disabilities != null &&
                    input.Disabilities != null &&
                    this.Disabilities.SequenceEqual(input.Disabilities)
                ) && 
                (
                    this.DisplacementStatus == input.DisplacementStatus ||
                    (this.DisplacementStatus != null &&
                    this.DisplacementStatus.Equals(input.DisplacementStatus))
                ) && 
                (
                    this.EconomicDisadvantaged == input.EconomicDisadvantaged ||
                    this.EconomicDisadvantaged.Equals(input.EconomicDisadvantaged)
                ) && 
                (
                    this.ElectronicMails == input.ElectronicMails ||
                    this.ElectronicMails != null &&
                    input.ElectronicMails != null &&
                    this.ElectronicMails.SequenceEqual(input.ElectronicMails)
                ) && 
                (
                    this.EnglishLanguageExamDescriptor == input.EnglishLanguageExamDescriptor ||
                    (this.EnglishLanguageExamDescriptor != null &&
                    this.EnglishLanguageExamDescriptor.Equals(input.EnglishLanguageExamDescriptor))
                ) && 
                (
                    this.FirstGenerationStudent == input.FirstGenerationStudent ||
                    this.FirstGenerationStudent.Equals(input.FirstGenerationStudent)
                ) && 
                (
                    this.FirstName == input.FirstName ||
                    (this.FirstName != null &&
                    this.FirstName.Equals(input.FirstName))
                ) && 
                (
                    this.GenderDescriptor == input.GenderDescriptor ||
                    (this.GenderDescriptor != null &&
                    this.GenderDescriptor.Equals(input.GenderDescriptor))
                ) && 
                (
                    this.GenerationCodeSuffix == input.GenerationCodeSuffix ||
                    (this.GenerationCodeSuffix != null &&
                    this.GenerationCodeSuffix.Equals(input.GenerationCodeSuffix))
                ) && 
                (
                    this.HispanicLatinoEthnicity == input.HispanicLatinoEthnicity ||
                    this.HispanicLatinoEthnicity.Equals(input.HispanicLatinoEthnicity)
                ) && 
                (
                    this.Languages == input.Languages ||
                    this.Languages != null &&
                    input.Languages != null &&
                    this.Languages.SequenceEqual(input.Languages)
                ) && 
                (
                    this.LastSurname == input.LastSurname ||
                    (this.LastSurname != null &&
                    this.LastSurname.Equals(input.LastSurname))
                ) && 
                (
                    this.LimitedEnglishProficiencyDescriptor == input.LimitedEnglishProficiencyDescriptor ||
                    (this.LimitedEnglishProficiencyDescriptor != null &&
                    this.LimitedEnglishProficiencyDescriptor.Equals(input.LimitedEnglishProficiencyDescriptor))
                ) && 
                (
                    this.MaidenName == input.MaidenName ||
                    (this.MaidenName != null &&
                    this.MaidenName.Equals(input.MaidenName))
                ) && 
                (
                    this.MiddleName == input.MiddleName ||
                    (this.MiddleName != null &&
                    this.MiddleName.Equals(input.MiddleName))
                ) && 
                (
                    this.MultipleBirthStatus == input.MultipleBirthStatus ||
                    this.MultipleBirthStatus.Equals(input.MultipleBirthStatus)
                ) && 
                (
                    this.OtherNames == input.OtherNames ||
                    this.OtherNames != null &&
                    input.OtherNames != null &&
                    this.OtherNames.SequenceEqual(input.OtherNames)
                ) && 
                (
                    this.PersonalIdentificationDocuments == input.PersonalIdentificationDocuments ||
                    this.PersonalIdentificationDocuments != null &&
                    input.PersonalIdentificationDocuments != null &&
                    this.PersonalIdentificationDocuments.SequenceEqual(input.PersonalIdentificationDocuments)
                ) && 
                (
                    this.PersonalTitlePrefix == input.PersonalTitlePrefix ||
                    (this.PersonalTitlePrefix != null &&
                    this.PersonalTitlePrefix.Equals(input.PersonalTitlePrefix))
                ) && 
                (
                    this.Races == input.Races ||
                    this.Races != null &&
                    input.Races != null &&
                    this.Races.SequenceEqual(input.Races)
                ) && 
                (
                    this.SexDescriptor == input.SexDescriptor ||
                    (this.SexDescriptor != null &&
                    this.SexDescriptor.Equals(input.SexDescriptor))
                ) && 
                (
                    this.Telephones == input.Telephones ||
                    this.Telephones != null &&
                    input.Telephones != null &&
                    this.Telephones.SequenceEqual(input.Telephones)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.CandidateIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.CandidateIdentifier.GetHashCode();
                }
                if (this.PersonReference != null)
                {
                    hashCode = (hashCode * 59) + this.PersonReference.GetHashCode();
                }
                if (this.Addresses != null)
                {
                    hashCode = (hashCode * 59) + this.Addresses.GetHashCode();
                }
                if (this.BirthCity != null)
                {
                    hashCode = (hashCode * 59) + this.BirthCity.GetHashCode();
                }
                if (this.BirthCountryDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.BirthCountryDescriptor.GetHashCode();
                }
                if (this.BirthDate != null)
                {
                    hashCode = (hashCode * 59) + this.BirthDate.GetHashCode();
                }
                if (this.BirthInternationalProvince != null)
                {
                    hashCode = (hashCode * 59) + this.BirthInternationalProvince.GetHashCode();
                }
                if (this.BirthSexDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.BirthSexDescriptor.GetHashCode();
                }
                if (this.BirthStateAbbreviationDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.BirthStateAbbreviationDescriptor.GetHashCode();
                }
                if (this.DateEnteredUS != null)
                {
                    hashCode = (hashCode * 59) + this.DateEnteredUS.GetHashCode();
                }
                if (this.Disabilities != null)
                {
                    hashCode = (hashCode * 59) + this.Disabilities.GetHashCode();
                }
                if (this.DisplacementStatus != null)
                {
                    hashCode = (hashCode * 59) + this.DisplacementStatus.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.EconomicDisadvantaged.GetHashCode();
                if (this.ElectronicMails != null)
                {
                    hashCode = (hashCode * 59) + this.ElectronicMails.GetHashCode();
                }
                if (this.EnglishLanguageExamDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.EnglishLanguageExamDescriptor.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.FirstGenerationStudent.GetHashCode();
                if (this.FirstName != null)
                {
                    hashCode = (hashCode * 59) + this.FirstName.GetHashCode();
                }
                if (this.GenderDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.GenderDescriptor.GetHashCode();
                }
                if (this.GenerationCodeSuffix != null)
                {
                    hashCode = (hashCode * 59) + this.GenerationCodeSuffix.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.HispanicLatinoEthnicity.GetHashCode();
                if (this.Languages != null)
                {
                    hashCode = (hashCode * 59) + this.Languages.GetHashCode();
                }
                if (this.LastSurname != null)
                {
                    hashCode = (hashCode * 59) + this.LastSurname.GetHashCode();
                }
                if (this.LimitedEnglishProficiencyDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.LimitedEnglishProficiencyDescriptor.GetHashCode();
                }
                if (this.MaidenName != null)
                {
                    hashCode = (hashCode * 59) + this.MaidenName.GetHashCode();
                }
                if (this.MiddleName != null)
                {
                    hashCode = (hashCode * 59) + this.MiddleName.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.MultipleBirthStatus.GetHashCode();
                if (this.OtherNames != null)
                {
                    hashCode = (hashCode * 59) + this.OtherNames.GetHashCode();
                }
                if (this.PersonalIdentificationDocuments != null)
                {
                    hashCode = (hashCode * 59) + this.PersonalIdentificationDocuments.GetHashCode();
                }
                if (this.PersonalTitlePrefix != null)
                {
                    hashCode = (hashCode * 59) + this.PersonalTitlePrefix.GetHashCode();
                }
                if (this.Races != null)
                {
                    hashCode = (hashCode * 59) + this.Races.GetHashCode();
                }
                if (this.SexDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.SexDescriptor.GetHashCode();
                }
                if (this.Telephones != null)
                {
                    hashCode = (hashCode * 59) + this.Telephones.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // CandidateIdentifier (string) maxLength
            if (this.CandidateIdentifier != null && this.CandidateIdentifier.Length > 32)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CandidateIdentifier, length must be less than 32.", new [] { "CandidateIdentifier" });
            }

            // BirthCity (string) maxLength
            if (this.BirthCity != null && this.BirthCity.Length > 30)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BirthCity, length must be less than 30.", new [] { "BirthCity" });
            }

            // BirthCountryDescriptor (string) maxLength
            if (this.BirthCountryDescriptor != null && this.BirthCountryDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BirthCountryDescriptor, length must be less than 306.", new [] { "BirthCountryDescriptor" });
            }

            // BirthInternationalProvince (string) maxLength
            if (this.BirthInternationalProvince != null && this.BirthInternationalProvince.Length > 150)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BirthInternationalProvince, length must be less than 150.", new [] { "BirthInternationalProvince" });
            }

            // BirthSexDescriptor (string) maxLength
            if (this.BirthSexDescriptor != null && this.BirthSexDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BirthSexDescriptor, length must be less than 306.", new [] { "BirthSexDescriptor" });
            }

            // BirthStateAbbreviationDescriptor (string) maxLength
            if (this.BirthStateAbbreviationDescriptor != null && this.BirthStateAbbreviationDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BirthStateAbbreviationDescriptor, length must be less than 306.", new [] { "BirthStateAbbreviationDescriptor" });
            }

            // DisplacementStatus (string) maxLength
            if (this.DisplacementStatus != null && this.DisplacementStatus.Length > 30)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DisplacementStatus, length must be less than 30.", new [] { "DisplacementStatus" });
            }

            // EnglishLanguageExamDescriptor (string) maxLength
            if (this.EnglishLanguageExamDescriptor != null && this.EnglishLanguageExamDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for EnglishLanguageExamDescriptor, length must be less than 306.", new [] { "EnglishLanguageExamDescriptor" });
            }

            // FirstName (string) maxLength
            if (this.FirstName != null && this.FirstName.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for FirstName, length must be less than 75.", new [] { "FirstName" });
            }

            // GenderDescriptor (string) maxLength
            if (this.GenderDescriptor != null && this.GenderDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for GenderDescriptor, length must be less than 306.", new [] { "GenderDescriptor" });
            }

            // GenerationCodeSuffix (string) maxLength
            if (this.GenerationCodeSuffix != null && this.GenerationCodeSuffix.Length > 10)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for GenerationCodeSuffix, length must be less than 10.", new [] { "GenerationCodeSuffix" });
            }

            // LastSurname (string) maxLength
            if (this.LastSurname != null && this.LastSurname.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for LastSurname, length must be less than 75.", new [] { "LastSurname" });
            }

            // LimitedEnglishProficiencyDescriptor (string) maxLength
            if (this.LimitedEnglishProficiencyDescriptor != null && this.LimitedEnglishProficiencyDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for LimitedEnglishProficiencyDescriptor, length must be less than 306.", new [] { "LimitedEnglishProficiencyDescriptor" });
            }

            // MaidenName (string) maxLength
            if (this.MaidenName != null && this.MaidenName.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for MaidenName, length must be less than 75.", new [] { "MaidenName" });
            }

            // MiddleName (string) maxLength
            if (this.MiddleName != null && this.MiddleName.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for MiddleName, length must be less than 75.", new [] { "MiddleName" });
            }

            // PersonalTitlePrefix (string) maxLength
            if (this.PersonalTitlePrefix != null && this.PersonalTitlePrefix.Length > 30)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for PersonalTitlePrefix, length must be less than 30.", new [] { "PersonalTitlePrefix" });
            }

            // SexDescriptor (string) maxLength
            if (this.SexDescriptor != null && this.SexDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SexDescriptor, length must be less than 306.", new [] { "SexDescriptor" });
            }

            yield break;
        }
    }

}
